# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: proofs.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='proofs.proto',
  package='ics23',
  syntax='proto3',
  serialized_options=b'Z\032github.com/confio/ics23/go',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x0cproofs.proto\x12\x05ics23\"g\n\x0e\x45xistenceProof\x12\x0b\n\x03key\x18\x01 \x01(\x0c\x12\r\n\x05value\x18\x02 \x01(\x0c\x12\x1b\n\x04leaf\x18\x03 \x01(\x0b\x32\r.ics23.LeafOp\x12\x1c\n\x04path\x18\x04 \x03(\x0b\x32\x0e.ics23.InnerOp\"k\n\x11NonExistenceProof\x12\x0b\n\x03key\x18\x01 \x01(\x0c\x12#\n\x04left\x18\x02 \x01(\x0b\x32\x15.ics23.ExistenceProof\x12$\n\x05right\x18\x03 \x01(\x0b\x32\x15.ics23.ExistenceProof\"\xc7\x01\n\x0f\x43ommitmentProof\x12&\n\x05\x65xist\x18\x01 \x01(\x0b\x32\x15.ics23.ExistenceProofH\x00\x12,\n\x08nonexist\x18\x02 \x01(\x0b\x32\x18.ics23.NonExistenceProofH\x00\x12\"\n\x05\x62\x61tch\x18\x03 \x01(\x0b\x32\x11.ics23.BatchProofH\x00\x12\x31\n\ncompressed\x18\x04 \x01(\x0b\x32\x1b.ics23.CompressedBatchProofH\x00\x42\x07\n\x05proof\"\xa0\x01\n\x06LeafOp\x12\x1b\n\x04hash\x18\x01 \x01(\x0e\x32\r.ics23.HashOp\x12\"\n\x0bprehash_key\x18\x02 \x01(\x0e\x32\r.ics23.HashOp\x12$\n\rprehash_value\x18\x03 \x01(\x0e\x32\r.ics23.HashOp\x12\x1f\n\x06length\x18\x04 \x01(\x0e\x32\x0f.ics23.LengthOp\x12\x0e\n\x06prefix\x18\x05 \x01(\x0c\"F\n\x07InnerOp\x12\x1b\n\x04hash\x18\x01 \x01(\x0e\x32\r.ics23.HashOp\x12\x0e\n\x06prefix\x18\x02 \x01(\x0c\x12\x0e\n\x06suffix\x18\x03 \x01(\x0c\"y\n\tProofSpec\x12 \n\tleaf_spec\x18\x01 \x01(\x0b\x32\r.ics23.LeafOp\x12$\n\ninner_spec\x18\x02 \x01(\x0b\x32\x10.ics23.InnerSpec\x12\x11\n\tmax_depth\x18\x03 \x01(\x05\x12\x11\n\tmin_depth\x18\x04 \x01(\x05\"\x9c\x01\n\tInnerSpec\x12\x13\n\x0b\x63hild_order\x18\x01 \x03(\x05\x12\x12\n\nchild_size\x18\x02 \x01(\x05\x12\x19\n\x11min_prefix_length\x18\x03 \x01(\x05\x12\x19\n\x11max_prefix_length\x18\x04 \x01(\x05\x12\x13\n\x0b\x65mpty_child\x18\x05 \x01(\x0c\x12\x1b\n\x04hash\x18\x06 \x01(\x0e\x32\r.ics23.HashOp\"0\n\nBatchProof\x12\"\n\x07\x65ntries\x18\x01 \x03(\x0b\x32\x11.ics23.BatchEntry\"k\n\nBatchEntry\x12&\n\x05\x65xist\x18\x01 \x01(\x0b\x32\x15.ics23.ExistenceProofH\x00\x12,\n\x08nonexist\x18\x02 \x01(\x0b\x32\x18.ics23.NonExistenceProofH\x00\x42\x07\n\x05proof\"k\n\x14\x43ompressedBatchProof\x12,\n\x07\x65ntries\x18\x01 \x03(\x0b\x32\x1b.ics23.CompressedBatchEntry\x12%\n\rlookup_inners\x18\x02 \x03(\x0b\x32\x0e.ics23.InnerOp\"\x89\x01\n\x14\x43ompressedBatchEntry\x12\x30\n\x05\x65xist\x18\x01 \x01(\x0b\x32\x1f.ics23.CompressedExistenceProofH\x00\x12\x36\n\x08nonexist\x18\x02 \x01(\x0b\x32\".ics23.CompressedNonExistenceProofH\x00\x42\x07\n\x05proof\"a\n\x18\x43ompressedExistenceProof\x12\x0b\n\x03key\x18\x01 \x01(\x0c\x12\r\n\x05value\x18\x02 \x01(\x0c\x12\x1b\n\x04leaf\x18\x03 \x01(\x0b\x32\r.ics23.LeafOp\x12\x0c\n\x04path\x18\x04 \x03(\x05\"\x89\x01\n\x1b\x43ompressedNonExistenceProof\x12\x0b\n\x03key\x18\x01 \x01(\x0c\x12-\n\x04left\x18\x02 \x01(\x0b\x32\x1f.ics23.CompressedExistenceProof\x12.\n\x05right\x18\x03 \x01(\x0b\x32\x1f.ics23.CompressedExistenceProof*U\n\x06HashOp\x12\x0b\n\x07NO_HASH\x10\x00\x12\n\n\x06SHA256\x10\x01\x12\n\n\x06SHA512\x10\x02\x12\n\n\x06KECCAK\x10\x03\x12\r\n\tRIPEMD160\x10\x04\x12\x0b\n\x07\x42ITCOIN\x10\x05*\xab\x01\n\x08LengthOp\x12\r\n\tNO_PREFIX\x10\x00\x12\r\n\tVAR_PROTO\x10\x01\x12\x0b\n\x07VAR_RLP\x10\x02\x12\x0f\n\x0b\x46IXED32_BIG\x10\x03\x12\x12\n\x0e\x46IXED32_LITTLE\x10\x04\x12\x0f\n\x0b\x46IXED64_BIG\x10\x05\x12\x12\n\x0e\x46IXED64_LITTLE\x10\x06\x12\x14\n\x10REQUIRE_32_BYTES\x10\x07\x12\x14\n\x10REQUIRE_64_BYTES\x10\x08\x42\x1cZ\x1agithub.com/confio/ics23/gob\x06proto3'
)

_HASHOP = _descriptor.EnumDescriptor(
  name='HashOp',
  full_name='ics23.HashOp',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='NO_HASH', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SHA256', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SHA512', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='KECCAK', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='RIPEMD160', index=4, number=4,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='BITCOIN', index=5, number=5,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1603,
  serialized_end=1688,
)
_sym_db.RegisterEnumDescriptor(_HASHOP)

HashOp = enum_type_wrapper.EnumTypeWrapper(_HASHOP)
_LENGTHOP = _descriptor.EnumDescriptor(
  name='LengthOp',
  full_name='ics23.LengthOp',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='NO_PREFIX', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='VAR_PROTO', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='VAR_RLP', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='FIXED32_BIG', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='FIXED32_LITTLE', index=4, number=4,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='FIXED64_BIG', index=5, number=5,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='FIXED64_LITTLE', index=6, number=6,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='REQUIRE_32_BYTES', index=7, number=7,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='REQUIRE_64_BYTES', index=8, number=8,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1691,
  serialized_end=1862,
)
_sym_db.RegisterEnumDescriptor(_LENGTHOP)

LengthOp = enum_type_wrapper.EnumTypeWrapper(_LENGTHOP)
NO_HASH = 0
SHA256 = 1
SHA512 = 2
KECCAK = 3
RIPEMD160 = 4
BITCOIN = 5
NO_PREFIX = 0
VAR_PROTO = 1
VAR_RLP = 2
FIXED32_BIG = 3
FIXED32_LITTLE = 4
FIXED64_BIG = 5
FIXED64_LITTLE = 6
REQUIRE_32_BYTES = 7
REQUIRE_64_BYTES = 8



_EXISTENCEPROOF = _descriptor.Descriptor(
  name='ExistenceProof',
  full_name='ics23.ExistenceProof',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='ics23.ExistenceProof.key', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='ics23.ExistenceProof.value', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='leaf', full_name='ics23.ExistenceProof.leaf', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='path', full_name='ics23.ExistenceProof.path', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=23,
  serialized_end=126,
)


_NONEXISTENCEPROOF = _descriptor.Descriptor(
  name='NonExistenceProof',
  full_name='ics23.NonExistenceProof',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='ics23.NonExistenceProof.key', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='left', full_name='ics23.NonExistenceProof.left', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='right', full_name='ics23.NonExistenceProof.right', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=128,
  serialized_end=235,
)


_COMMITMENTPROOF = _descriptor.Descriptor(
  name='CommitmentProof',
  full_name='ics23.CommitmentProof',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='exist', full_name='ics23.CommitmentProof.exist', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='nonexist', full_name='ics23.CommitmentProof.nonexist', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='batch', full_name='ics23.CommitmentProof.batch', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='compressed', full_name='ics23.CommitmentProof.compressed', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='proof', full_name='ics23.CommitmentProof.proof',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=238,
  serialized_end=437,
)


_LEAFOP = _descriptor.Descriptor(
  name='LeafOp',
  full_name='ics23.LeafOp',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='hash', full_name='ics23.LeafOp.hash', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='prehash_key', full_name='ics23.LeafOp.prehash_key', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='prehash_value', full_name='ics23.LeafOp.prehash_value', index=2,
      number=3, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='length', full_name='ics23.LeafOp.length', index=3,
      number=4, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='prefix', full_name='ics23.LeafOp.prefix', index=4,
      number=5, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=440,
  serialized_end=600,
)


_INNEROP = _descriptor.Descriptor(
  name='InnerOp',
  full_name='ics23.InnerOp',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='hash', full_name='ics23.InnerOp.hash', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='prefix', full_name='ics23.InnerOp.prefix', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='suffix', full_name='ics23.InnerOp.suffix', index=2,
      number=3, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=602,
  serialized_end=672,
)


_PROOFSPEC = _descriptor.Descriptor(
  name='ProofSpec',
  full_name='ics23.ProofSpec',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='leaf_spec', full_name='ics23.ProofSpec.leaf_spec', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='inner_spec', full_name='ics23.ProofSpec.inner_spec', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='max_depth', full_name='ics23.ProofSpec.max_depth', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='min_depth', full_name='ics23.ProofSpec.min_depth', index=3,
      number=4, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=674,
  serialized_end=795,
)


_INNERSPEC = _descriptor.Descriptor(
  name='InnerSpec',
  full_name='ics23.InnerSpec',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='child_order', full_name='ics23.InnerSpec.child_order', index=0,
      number=1, type=5, cpp_type=1, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='child_size', full_name='ics23.InnerSpec.child_size', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='min_prefix_length', full_name='ics23.InnerSpec.min_prefix_length', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='max_prefix_length', full_name='ics23.InnerSpec.max_prefix_length', index=3,
      number=4, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='empty_child', full_name='ics23.InnerSpec.empty_child', index=4,
      number=5, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='hash', full_name='ics23.InnerSpec.hash', index=5,
      number=6, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=798,
  serialized_end=954,
)


_BATCHPROOF = _descriptor.Descriptor(
  name='BatchProof',
  full_name='ics23.BatchProof',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='entries', full_name='ics23.BatchProof.entries', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=956,
  serialized_end=1004,
)


_BATCHENTRY = _descriptor.Descriptor(
  name='BatchEntry',
  full_name='ics23.BatchEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='exist', full_name='ics23.BatchEntry.exist', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='nonexist', full_name='ics23.BatchEntry.nonexist', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='proof', full_name='ics23.BatchEntry.proof',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=1006,
  serialized_end=1113,
)


_COMPRESSEDBATCHPROOF = _descriptor.Descriptor(
  name='CompressedBatchProof',
  full_name='ics23.CompressedBatchProof',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='entries', full_name='ics23.CompressedBatchProof.entries', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='lookup_inners', full_name='ics23.CompressedBatchProof.lookup_inners', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1115,
  serialized_end=1222,
)


_COMPRESSEDBATCHENTRY = _descriptor.Descriptor(
  name='CompressedBatchEntry',
  full_name='ics23.CompressedBatchEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='exist', full_name='ics23.CompressedBatchEntry.exist', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='nonexist', full_name='ics23.CompressedBatchEntry.nonexist', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='proof', full_name='ics23.CompressedBatchEntry.proof',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=1225,
  serialized_end=1362,
)


_COMPRESSEDEXISTENCEPROOF = _descriptor.Descriptor(
  name='CompressedExistenceProof',
  full_name='ics23.CompressedExistenceProof',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='ics23.CompressedExistenceProof.key', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='ics23.CompressedExistenceProof.value', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='leaf', full_name='ics23.CompressedExistenceProof.leaf', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='path', full_name='ics23.CompressedExistenceProof.path', index=3,
      number=4, type=5, cpp_type=1, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1364,
  serialized_end=1461,
)


_COMPRESSEDNONEXISTENCEPROOF = _descriptor.Descriptor(
  name='CompressedNonExistenceProof',
  full_name='ics23.CompressedNonExistenceProof',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='ics23.CompressedNonExistenceProof.key', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='left', full_name='ics23.CompressedNonExistenceProof.left', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='right', full_name='ics23.CompressedNonExistenceProof.right', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1464,
  serialized_end=1601,
)

_EXISTENCEPROOF.fields_by_name['leaf'].message_type = _LEAFOP
_EXISTENCEPROOF.fields_by_name['path'].message_type = _INNEROP
_NONEXISTENCEPROOF.fields_by_name['left'].message_type = _EXISTENCEPROOF
_NONEXISTENCEPROOF.fields_by_name['right'].message_type = _EXISTENCEPROOF
_COMMITMENTPROOF.fields_by_name['exist'].message_type = _EXISTENCEPROOF
_COMMITMENTPROOF.fields_by_name['nonexist'].message_type = _NONEXISTENCEPROOF
_COMMITMENTPROOF.fields_by_name['batch'].message_type = _BATCHPROOF
_COMMITMENTPROOF.fields_by_name['compressed'].message_type = _COMPRESSEDBATCHPROOF
_COMMITMENTPROOF.oneofs_by_name['proof'].fields.append(
  _COMMITMENTPROOF.fields_by_name['exist'])
_COMMITMENTPROOF.fields_by_name['exist'].containing_oneof = _COMMITMENTPROOF.oneofs_by_name['proof']
_COMMITMENTPROOF.oneofs_by_name['proof'].fields.append(
  _COMMITMENTPROOF.fields_by_name['nonexist'])
_COMMITMENTPROOF.fields_by_name['nonexist'].containing_oneof = _COMMITMENTPROOF.oneofs_by_name['proof']
_COMMITMENTPROOF.oneofs_by_name['proof'].fields.append(
  _COMMITMENTPROOF.fields_by_name['batch'])
_COMMITMENTPROOF.fields_by_name['batch'].containing_oneof = _COMMITMENTPROOF.oneofs_by_name['proof']
_COMMITMENTPROOF.oneofs_by_name['proof'].fields.append(
  _COMMITMENTPROOF.fields_by_name['compressed'])
_COMMITMENTPROOF.fields_by_name['compressed'].containing_oneof = _COMMITMENTPROOF.oneofs_by_name['proof']
_LEAFOP.fields_by_name['hash'].enum_type = _HASHOP
_LEAFOP.fields_by_name['prehash_key'].enum_type = _HASHOP
_LEAFOP.fields_by_name['prehash_value'].enum_type = _HASHOP
_LEAFOP.fields_by_name['length'].enum_type = _LENGTHOP
_INNEROP.fields_by_name['hash'].enum_type = _HASHOP
_PROOFSPEC.fields_by_name['leaf_spec'].message_type = _LEAFOP
_PROOFSPEC.fields_by_name['inner_spec'].message_type = _INNERSPEC
_INNERSPEC.fields_by_name['hash'].enum_type = _HASHOP
_BATCHPROOF.fields_by_name['entries'].message_type = _BATCHENTRY
_BATCHENTRY.fields_by_name['exist'].message_type = _EXISTENCEPROOF
_BATCHENTRY.fields_by_name['nonexist'].message_type = _NONEXISTENCEPROOF
_BATCHENTRY.oneofs_by_name['proof'].fields.append(
  _BATCHENTRY.fields_by_name['exist'])
_BATCHENTRY.fields_by_name['exist'].containing_oneof = _BATCHENTRY.oneofs_by_name['proof']
_BATCHENTRY.oneofs_by_name['proof'].fields.append(
  _BATCHENTRY.fields_by_name['nonexist'])
_BATCHENTRY.fields_by_name['nonexist'].containing_oneof = _BATCHENTRY.oneofs_by_name['proof']
_COMPRESSEDBATCHPROOF.fields_by_name['entries'].message_type = _COMPRESSEDBATCHENTRY
_COMPRESSEDBATCHPROOF.fields_by_name['lookup_inners'].message_type = _INNEROP
_COMPRESSEDBATCHENTRY.fields_by_name['exist'].message_type = _COMPRESSEDEXISTENCEPROOF
_COMPRESSEDBATCHENTRY.fields_by_name['nonexist'].message_type = _COMPRESSEDNONEXISTENCEPROOF
_COMPRESSEDBATCHENTRY.oneofs_by_name['proof'].fields.append(
  _COMPRESSEDBATCHENTRY.fields_by_name['exist'])
_COMPRESSEDBATCHENTRY.fields_by_name['exist'].containing_oneof = _COMPRESSEDBATCHENTRY.oneofs_by_name['proof']
_COMPRESSEDBATCHENTRY.oneofs_by_name['proof'].fields.append(
  _COMPRESSEDBATCHENTRY.fields_by_name['nonexist'])
_COMPRESSEDBATCHENTRY.fields_by_name['nonexist'].containing_oneof = _COMPRESSEDBATCHENTRY.oneofs_by_name['proof']
_COMPRESSEDEXISTENCEPROOF.fields_by_name['leaf'].message_type = _LEAFOP
_COMPRESSEDNONEXISTENCEPROOF.fields_by_name['left'].message_type = _COMPRESSEDEXISTENCEPROOF
_COMPRESSEDNONEXISTENCEPROOF.fields_by_name['right'].message_type = _COMPRESSEDEXISTENCEPROOF
DESCRIPTOR.message_types_by_name['ExistenceProof'] = _EXISTENCEPROOF
DESCRIPTOR.message_types_by_name['NonExistenceProof'] = _NONEXISTENCEPROOF
DESCRIPTOR.message_types_by_name['CommitmentProof'] = _COMMITMENTPROOF
DESCRIPTOR.message_types_by_name['LeafOp'] = _LEAFOP
DESCRIPTOR.message_types_by_name['InnerOp'] = _INNEROP
DESCRIPTOR.message_types_by_name['ProofSpec'] = _PROOFSPEC
DESCRIPTOR.message_types_by_name['InnerSpec'] = _INNERSPEC
DESCRIPTOR.message_types_by_name['BatchProof'] = _BATCHPROOF
DESCRIPTOR.message_types_by_name['BatchEntry'] = _BATCHENTRY
DESCRIPTOR.message_types_by_name['CompressedBatchProof'] = _COMPRESSEDBATCHPROOF
DESCRIPTOR.message_types_by_name['CompressedBatchEntry'] = _COMPRESSEDBATCHENTRY
DESCRIPTOR.message_types_by_name['CompressedExistenceProof'] = _COMPRESSEDEXISTENCEPROOF
DESCRIPTOR.message_types_by_name['CompressedNonExistenceProof'] = _COMPRESSEDNONEXISTENCEPROOF
DESCRIPTOR.enum_types_by_name['HashOp'] = _HASHOP
DESCRIPTOR.enum_types_by_name['LengthOp'] = _LENGTHOP
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

ExistenceProof = _reflection.GeneratedProtocolMessageType('ExistenceProof', (_message.Message,), {
  'DESCRIPTOR' : _EXISTENCEPROOF,
  '__module__' : 'proofs_pb2'
  # @@protoc_insertion_point(class_scope:ics23.ExistenceProof)
  })
_sym_db.RegisterMessage(ExistenceProof)

NonExistenceProof = _reflection.GeneratedProtocolMessageType('NonExistenceProof', (_message.Message,), {
  'DESCRIPTOR' : _NONEXISTENCEPROOF,
  '__module__' : 'proofs_pb2'
  # @@protoc_insertion_point(class_scope:ics23.NonExistenceProof)
  })
_sym_db.RegisterMessage(NonExistenceProof)

CommitmentProof = _reflection.GeneratedProtocolMessageType('CommitmentProof', (_message.Message,), {
  'DESCRIPTOR' : _COMMITMENTPROOF,
  '__module__' : 'proofs_pb2'
  # @@protoc_insertion_point(class_scope:ics23.CommitmentProof)
  })
_sym_db.RegisterMessage(CommitmentProof)

LeafOp = _reflection.GeneratedProtocolMessageType('LeafOp', (_message.Message,), {
  'DESCRIPTOR' : _LEAFOP,
  '__module__' : 'proofs_pb2'
  # @@protoc_insertion_point(class_scope:ics23.LeafOp)
  })
_sym_db.RegisterMessage(LeafOp)

InnerOp = _reflection.GeneratedProtocolMessageType('InnerOp', (_message.Message,), {
  'DESCRIPTOR' : _INNEROP,
  '__module__' : 'proofs_pb2'
  # @@protoc_insertion_point(class_scope:ics23.InnerOp)
  })
_sym_db.RegisterMessage(InnerOp)

ProofSpec = _reflection.GeneratedProtocolMessageType('ProofSpec', (_message.Message,), {
  'DESCRIPTOR' : _PROOFSPEC,
  '__module__' : 'proofs_pb2'
  # @@protoc_insertion_point(class_scope:ics23.ProofSpec)
  })
_sym_db.RegisterMessage(ProofSpec)

InnerSpec = _reflection.GeneratedProtocolMessageType('InnerSpec', (_message.Message,), {
  'DESCRIPTOR' : _INNERSPEC,
  '__module__' : 'proofs_pb2'
  # @@protoc_insertion_point(class_scope:ics23.InnerSpec)
  })
_sym_db.RegisterMessage(InnerSpec)

BatchProof = _reflection.GeneratedProtocolMessageType('BatchProof', (_message.Message,), {
  'DESCRIPTOR' : _BATCHPROOF,
  '__module__' : 'proofs_pb2'
  # @@protoc_insertion_point(class_scope:ics23.BatchProof)
  })
_sym_db.RegisterMessage(BatchProof)

BatchEntry = _reflection.GeneratedProtocolMessageType('BatchEntry', (_message.Message,), {
  'DESCRIPTOR' : _BATCHENTRY,
  '__module__' : 'proofs_pb2'
  # @@protoc_insertion_point(class_scope:ics23.BatchEntry)
  })
_sym_db.RegisterMessage(BatchEntry)

CompressedBatchProof = _reflection.GeneratedProtocolMessageType('CompressedBatchProof', (_message.Message,), {
  'DESCRIPTOR' : _COMPRESSEDBATCHPROOF,
  '__module__' : 'proofs_pb2'
  # @@protoc_insertion_point(class_scope:ics23.CompressedBatchProof)
  })
_sym_db.RegisterMessage(CompressedBatchProof)

CompressedBatchEntry = _reflection.GeneratedProtocolMessageType('CompressedBatchEntry', (_message.Message,), {
  'DESCRIPTOR' : _COMPRESSEDBATCHENTRY,
  '__module__' : 'proofs_pb2'
  # @@protoc_insertion_point(class_scope:ics23.CompressedBatchEntry)
  })
_sym_db.RegisterMessage(CompressedBatchEntry)

CompressedExistenceProof = _reflection.GeneratedProtocolMessageType('CompressedExistenceProof', (_message.Message,), {
  'DESCRIPTOR' : _COMPRESSEDEXISTENCEPROOF,
  '__module__' : 'proofs_pb2'
  # @@protoc_insertion_point(class_scope:ics23.CompressedExistenceProof)
  })
_sym_db.RegisterMessage(CompressedExistenceProof)

CompressedNonExistenceProof = _reflection.GeneratedProtocolMessageType('CompressedNonExistenceProof', (_message.Message,), {
  'DESCRIPTOR' : _COMPRESSEDNONEXISTENCEPROOF,
  '__module__' : 'proofs_pb2'
  # @@protoc_insertion_point(class_scope:ics23.CompressedNonExistenceProof)
  })
_sym_db.RegisterMessage(CompressedNonExistenceProof)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2018-2021 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------
class QueryError(RuntimeError):
    pass


class NotFoundError(QueryError):
    pass


class QueryTimeoutError(QueryError):
    pass


class BroadcastError(RuntimeError):
    def __init__(self, tx_hash: str, message: str):
        super().__init__(message)
        self.tx_hash = tx_hash


class OutOfGasError(BroadcastError):
    def __init__(self, tx_hash: str, gas_wanted: int, gas_used: int):
        self.gas_wanted = gas_wanted
        self.gas_used = gas_used
        super().__init__(
            tx_hash, f"Out of Gas (wanted: {self.gas_wanted}, used: {self.gas_used})"
        )


class InsufficientFeesError(BroadcastError):
    def __init__(self, tx_hash: str, minimum_required_fee: str):
        self.minimum_required_fee = minimum_required_fee
        super().__init__(
            tx_hash,
            f"Insufficient Fees (minimum required: {self.minimum_required_fee})",
        )



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ibc/applications/transfer/v1/transfer.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from gogoproto import gogo_pb2 as gogoproto_dot_gogo__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='ibc/applications/transfer/v1/transfer.proto',
  package='ibc.applications.transfer.v1',
  syntax='proto3',
  serialized_options=b'Z7github.com/cosmos/ibc-go/v2/modules/apps/transfer/types',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n+ibc/applications/transfer/v1/transfer.proto\x12\x1cibc.applications.transfer.v1\x1a\x14gogoproto/gogo.proto\".\n\nDenomTrace\x12\x0c\n\x04path\x18\x01 \x01(\t\x12\x12\n\nbase_denom\x18\x02 \x01(\t\"l\n\x06Params\x12-\n\x0csend_enabled\x18\x01 \x01(\x08\x42\x17\xf2\xde\x1f\x13yaml:\"send_enabled\"\x12\x33\n\x0freceive_enabled\x18\x02 \x01(\x08\x42\x1a\xf2\xde\x1f\x16yaml:\"receive_enabled\"B9Z7github.com/cosmos/ibc-go/v2/modules/apps/transfer/typesb\x06proto3'
  ,
  dependencies=[gogoproto_dot_gogo__pb2.DESCRIPTOR,])




_DENOMTRACE = _descriptor.Descriptor(
  name='DenomTrace',
  full_name='ibc.applications.transfer.v1.DenomTrace',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='path', full_name='ibc.applications.transfer.v1.DenomTrace.path', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='base_denom', full_name='ibc.applications.transfer.v1.DenomTrace.base_denom', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=99,
  serialized_end=145,
)


_PARAMS = _descriptor.Descriptor(
  name='Params',
  full_name='ibc.applications.transfer.v1.Params',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='send_enabled', full_name='ibc.applications.transfer.v1.Params.send_enabled', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\362\336\037\023yaml:\"send_enabled\"', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='receive_enabled', full_name='ibc.applications.transfer.v1.Params.receive_enabled', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\362\336\037\026yaml:\"receive_enabled\"', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=147,
  serialized_end=255,
)

DESCRIPTOR.message_types_by_name['DenomTrace'] = _DENOMTRACE
DESCRIPTOR.message_types_by_name['Params'] = _PARAMS
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

DenomTrace = _reflection.GeneratedProtocolMessageType('DenomTrace', (_message.Message,), {
  'DESCRIPTOR' : _DENOMTRACE,
  '__module__' : 'ibc.applications.transfer.v1.transfer_pb2'
  # @@protoc_insertion_point(class_scope:ibc.applications.transfer.v1.DenomTrace)
  })
_sym_db.RegisterMessage(DenomTrace)

Params = _reflection.GeneratedProtocolMessageType('Params', (_message.Message,), {
  'DESCRIPTOR' : _PARAMS,
  '__module__' : 'ibc.applications.transfer.v1.transfer_pb2'
  # @@protoc_insertion_point(class_scope:ibc.applications.transfer.v1.Params)
  })
_sym_db.RegisterMessage(Params)


DESCRIPTOR._options = None
_PARAMS.fields_by_name['send_enabled']._options = None
_PARAMS.fields_by_name['receive_enabled']._options = None
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2018-2021 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------
from cosmpy.aerial.client import LedgerClient, NetworkConfig
from cosmpy.aerial.wallet import LocalWallet
from cosmpy.crypto.keypairs import PrivateKey


def main():
    alice = LocalWallet(PrivateKey("X2Tv0Ok3RN2yi9GhWjLUX7RIfX5go9Wu+fwoJlqK2Og="))
    bob = LocalWallet(PrivateKey("p0h0sYImB4xGq3Zz+xfIrY4QR6CPqeNg8w6X3NUWLe4="))

    ledger = LedgerClient(NetworkConfig.fetchai_stable_testnet())

    print(
        f"Alice Address: {alice.address()} Balance: {ledger.query_bank_balance(alice.address())}"
    )
    print(
        f"Bob   Address: {bob.address()} Balance: {ledger.query_bank_balance(bob.address())}"
    )

    tx = ledger.send_tokens(bob.address(), 10, "atestfet", alice)

    print(f"TX {tx.tx_hash} waiting to complete...")
    tx.wait_to_complete()
    print(f"TX {tx.tx_hash} waiting to complete...done")

    print(
        f"Alice Address: {alice.address()} Balance: {ledger.query_bank_balance(alice.address())}"
    )
    print(
        f"Bob   Address: {bob.address()} Balance: {ledger.query_bank_balance(bob.address())}"
    )


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2018-2021 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------
import json
import time
from dataclasses import dataclass
from datetime import datetime, timedelta
from typing import Any, Dict, List, Optional, Tuple

import certifi
import grpc

from cosmpy.aerial.client.bank import create_bank_send_msg
from cosmpy.aerial.client.distribution import create_withdraw_delegator_reward
from cosmpy.aerial.client.staking import (
    ValidatorStatus,
    create_delegate_msg,
    create_redelegate_msg,
    create_undelegate_msg,
)
from cosmpy.aerial.client.utils import prepare_and_broadcast_basic_transaction
from cosmpy.aerial.config import NetworkConfig
from cosmpy.aerial.exceptions import NotFoundError, QueryTimeoutError
from cosmpy.aerial.gas import GasStrategy, SimulationGasStrategy
from cosmpy.aerial.tx import Transaction, TxState
from cosmpy.aerial.tx_helpers import MessageLog, SubmittedTx, TxResponse
from cosmpy.aerial.urls import Protocol, parse_url
from cosmpy.aerial.wallet import Wallet
from cosmpy.auth.rest_client import AuthRestClient
from cosmpy.bank.rest_client import BankRestClient
from cosmpy.common.rest_client import RestClient
from cosmpy.cosmwasm.rest_client import CosmWasmRestClient
from cosmpy.crypto.address import Address
from cosmpy.distribution.rest_client import DistributionRestClient
from cosmpy.params.rest_client import ParamsRestClient
from cosmpy.protos.cosmos.auth.v1beta1.auth_pb2 import BaseAccount
from cosmpy.protos.cosmos.auth.v1beta1.query_pb2 import QueryAccountRequest
from cosmpy.protos.cosmos.auth.v1beta1.query_pb2_grpc import QueryStub as AuthGrpcClient
from cosmpy.protos.cosmos.bank.v1beta1.query_pb2 import (
    QueryAllBalancesRequest,
    QueryBalanceRequest,
)
from cosmpy.protos.cosmos.bank.v1beta1.query_pb2_grpc import QueryStub as BankGrpcClient
from cosmpy.protos.cosmos.distribution.v1beta1.query_pb2 import (
    QueryDelegationRewardsRequest,
)
from cosmpy.protos.cosmos.distribution.v1beta1.query_pb2_grpc import (
    QueryStub as DistributionGrpcClient,
)
from cosmpy.protos.cosmos.params.v1beta1.query_pb2 import QueryParamsRequest
from cosmpy.protos.cosmos.params.v1beta1.query_pb2_grpc import (
    QueryStub as QueryParamsGrpcClient,
)
from cosmpy.protos.cosmos.staking.v1beta1.query_pb2 import (
    QueryDelegatorDelegationsRequest,
    QueryDelegatorUnbondingDelegationsRequest,
    QueryValidatorsRequest,
)
from cosmpy.protos.cosmos.staking.v1beta1.query_pb2_grpc import (
    QueryStub as StakingGrpcClient,
)
from cosmpy.protos.cosmos.tx.v1beta1.service_pb2 import (
    BroadcastMode,
    BroadcastTxRequest,
    GetTxRequest,
    SimulateRequest,
)
from cosmpy.protos.cosmos.tx.v1beta1.service_pb2_grpc import ServiceStub as TxGrpcClient
from cosmpy.protos.cosmwasm.wasm.v1.query_pb2_grpc import (
    QueryStub as CosmWasmGrpcClient,
)
from cosmpy.staking.rest_client import StakingRestClient
from cosmpy.tx.rest_client import TxRestClient

DEFAULT_QUERY_TIMEOUT_SECS = 15
DEFAULT_QUERY_INTERVAL_SECS = 2
COSMOS_SDK_DEC_COIN_PRECISION = 10**18


@dataclass
class Account:
    address: Address
    number: int
    sequence: int


@dataclass
class StakingPosition:
    validator: Address
    amount: int
    reward: int


@dataclass
class UnbondingPositions:
    validator: Address
    amount: int


@dataclass
class Validator:
    address: Address  # the operators address
    tokens: int  # The total amount of tokens for the validator
    moniker: str
    status: ValidatorStatus


@dataclass
class Coin:
    amount: int
    denom: str


@dataclass
class StakingSummary:
    current_positions: List[StakingPosition]
    unbonding_positions: List[UnbondingPositions]

    @property
    def total_staked(self) -> int:
        return sum(map(lambda p: p.amount, self.current_positions))

    @property
    def total_rewards(self) -> int:
        return sum(map(lambda p: p.reward, self.current_positions))

    @property
    def total_unbonding(self) -> int:
        return sum(map(lambda p: p.amount, self.unbonding_positions))


class LedgerClient:
    def __init__(self, cfg: NetworkConfig):
        cfg.validate()
        self._network_config = cfg
        self._gas_strategy: GasStrategy = SimulationGasStrategy(self)

        parsed_url = parse_url(cfg.url)

        if parsed_url.protocol == Protocol.GRPC:
            if parsed_url.secure:
                with open(certifi.where(), "rb") as f:
                    trusted_certs = f.read()
                credentials = grpc.ssl_channel_credentials(
                    root_certificates=trusted_certs
                )
                grpc_client = grpc.secure_channel(parsed_url.host_and_port, credentials)
            else:
                grpc_client = grpc.insecure_channel(parsed_url.host_and_port)

            self.wasm = CosmWasmGrpcClient(grpc_client)
            self.auth = AuthGrpcClient(grpc_client)
            self.txs = TxGrpcClient(grpc_client)
            self.bank = BankGrpcClient(grpc_client)
            self.staking = StakingGrpcClient(grpc_client)
            self.distribution = DistributionGrpcClient(grpc_client)
            self.params = QueryParamsGrpcClient(grpc_client)
        else:
            rest_client = RestClient(parsed_url.rest_url)

            self.wasm = CosmWasmRestClient(rest_client)  # type: ignore
            self.auth = AuthRestClient(rest_client)  # type: ignore
            self.txs = TxRestClient(rest_client)  # type: ignore
            self.bank = BankRestClient(rest_client)  # type: ignore
            self.staking = StakingRestClient(rest_client)  # type: ignore
            self.distribution = DistributionRestClient(rest_client)  # type: ignore
            self.params = ParamsRestClient(rest_client)  # type: ignore

    @property
    def network_config(self) -> NetworkConfig:
        return self._network_config

    @property
    def gas_strategy(self) -> GasStrategy:
        return self._gas_strategy

    @gas_strategy.setter
    def gas_strategy(self, strategy: GasStrategy):
        if not isinstance(strategy, GasStrategy):
            raise RuntimeError("Invalid strategy must implement GasStrategy interface")
        self._gas_strategy = strategy

    def query_account(self, address: Address) -> Account:
        request = QueryAccountRequest(address=str(address))
        response = self.auth.Account(request)

        account = BaseAccount()
        if not response.account.Is(BaseAccount.DESCRIPTOR):
            raise RuntimeError("Unexpected account type returned from query")
        response.account.Unpack(account)

        return Account(
            address=address,
            number=account.account_number,
            sequence=account.sequence,
        )

    def query_params(self, subspace: str, key: str) -> Any:
        req = QueryParamsRequest(subspace=subspace, key=key)
        resp = self.params.Params(req)
        return json.loads(resp.param.value)

    def query_bank_balance(self, address: Address, denom: Optional[str] = None) -> int:
        denom = denom or self.network_config.fee_denomination

        req = QueryBalanceRequest(
            address=str(address),
            denom=denom,
        )

        resp = self.bank.Balance(req)
        assert resp.balance.denom == denom  # sanity check

        return int(resp.balance.amount)

    def query_bank_all_balances(self, address: Address) -> List[Coin]:

        req = QueryAllBalancesRequest(address=str(address))
        resp = self.bank.AllBalances(req)

        return [Coin(amount=coin.amount, denom=coin.denom) for coin in resp.balances]

    def send_tokens(
        self,
        destination: Address,
        amount: int,
        denom: str,
        sender: Wallet,
        memo: Optional[str] = None,
        gas_limit: Optional[int] = None,
    ) -> SubmittedTx:

        # build up the store transaction
        tx = Transaction()
        tx.add_message(
            create_bank_send_msg(sender.address(), destination, amount, denom)
        )

        return prepare_and_broadcast_basic_transaction(
            self, tx, sender, gas_limit=gas_limit, memo=memo
        )

    def query_validators(
        self, status: Optional[ValidatorStatus] = None
    ) -> List[Validator]:
        filtered_status = status or ValidatorStatus.BONDED

        req = QueryValidatorsRequest()
        if filtered_status != ValidatorStatus.UNSPECIFIED:
            req.status = filtered_status.value

        resp = self.staking.Validators(req)

        validators: List[Validator] = []
        for validator in resp.validators:
            validators.append(
                Validator(
                    address=Address(validator.operator_address),
                    tokens=int(validator.tokens),
                    moniker=str(validator.description.moniker),
                    status=ValidatorStatus.from_proto(validator.status),
                )
            )
        return validators

    def query_staking_summary(self, address: Address) -> StakingSummary:
        current_positions: List[StakingPosition] = []

        req = QueryDelegatorDelegationsRequest(delegator_addr=str(address))
        resp = self.staking.DelegatorDelegations(req)

        # TODO: Need pagination support
        for item in resp.delegation_responses:

            req = QueryDelegationRewardsRequest(
                delegator_address=str(address),
                validator_address=str(item.delegation.validator_address),
            )
            rewards_resp = self.distribution.DelegationRewards(req)

            stake_reward = 0
            for reward in rewards_resp.rewards:
                if reward.denom == self.network_config.staking_denomination:
                    stake_reward = int(reward.amount) // COSMOS_SDK_DEC_COIN_PRECISION
                    break

            current_positions.append(
                StakingPosition(
                    validator=Address(item.delegation.validator_address),
                    amount=int(item.balance.amount),
                    reward=stake_reward,
                )
            )

        req = QueryDelegatorUnbondingDelegationsRequest(delegator_addr=str(address))
        resp = self.staking.DelegatorUnbondingDelegations(req)

        unbonding_summary: Dict[str, int] = {}
        for item in resp.unbonding_responses:
            validator = str(item.validator_address)
            total_unbonding = unbonding_summary.get(validator, 0)

            for entry in item.entries:
                total_unbonding += int(entry.balance)

            unbonding_summary[validator] = total_unbonding

        # build the final list of unbonding positions
        unbonding_positions: List[UnbondingPositions] = []
        for validator, total_unbonding in unbonding_summary.items():
            unbonding_positions.append(
                UnbondingPositions(
                    validator=Address(validator),
                    amount=total_unbonding,
                )
            )

        return StakingSummary(
            current_positions=current_positions, unbonding_positions=unbonding_positions
        )

    def delegate_tokens(
        self,
        validator: Address,
        amount: int,
        sender: Wallet,
        memo: Optional[str] = None,
        gas_limit: Optional[int] = None,
    ) -> SubmittedTx:
        tx = Transaction()
        tx.add_message(
            create_delegate_msg(
                sender.address(),
                validator,
                amount,
                self.network_config.staking_denomination,
            )
        )

        return prepare_and_broadcast_basic_transaction(
            self, tx, sender, gas_limit=gas_limit, memo=memo
        )

    def redelegate_tokens(
        self,
        current_validator: Address,
        next_validator: Address,
        amount: int,
        sender: Wallet,
        memo: Optional[str] = None,
        gas_limit: Optional[int] = None,
    ) -> SubmittedTx:
        tx = Transaction()
        tx.add_message(
            create_redelegate_msg(
                sender.address(),
                current_validator,
                next_validator,
                amount,
                self.network_config.staking_denomination,
            )
        )

        return prepare_and_broadcast_basic_transaction(
            self, tx, sender, gas_limit=gas_limit, memo=memo
        )

    def undelegate_tokens(
        self,
        validator: Address,
        amount: int,
        sender: Wallet,
        memo: Optional[str] = None,
        gas_limit: Optional[int] = None,
    ) -> SubmittedTx:

        tx = Transaction()
        tx.add_message(
            create_undelegate_msg(
                sender.address(),
                validator,
                amount,
                self.network_config.staking_denomination,
            )
        )

        return prepare_and_broadcast_basic_transaction(
            self, tx, sender, gas_limit=gas_limit, memo=memo
        )

    def claim_rewards(
        self,
        validator: Address,
        sender: Wallet,
        memo: Optional[str] = None,
        gas_limit: Optional[int] = None,
    ) -> SubmittedTx:
        tx = Transaction()
        tx.add_message(create_withdraw_delegator_reward(sender.address(), validator))

        return prepare_and_broadcast_basic_transaction(
            self, tx, sender, gas_limit=gas_limit, memo=memo
        )

    def estimate_gas_for_tx(self, tx: Transaction) -> int:
        return self._gas_strategy.estimate_gas(tx)

    def estimate_fee_from_gas(self, gas_limit: int) -> str:
        return f"{gas_limit * self.network_config.fee_minimum_gas_price}{self.network_config.fee_denomination}"

    def estimate_gas_and_fee_for_tx(self, tx: Transaction) -> Tuple[int, str]:
        gas_estimate = self.estimate_gas_for_tx(tx)
        fee = self.estimate_fee_from_gas(gas_estimate)
        return gas_estimate, fee

    def wait_for_query_tx(
        self,
        tx_hash: str,
        timeout: Optional[timedelta] = None,
        internal: Optional[timedelta] = None,
    ) -> TxResponse:
        timeout = timeout or timedelta(seconds=DEFAULT_QUERY_TIMEOUT_SECS)
        internal = internal or timedelta(seconds=DEFAULT_QUERY_INTERVAL_SECS)

        start = datetime.now()
        while True:
            try:
                return self.query_tx(tx_hash)
            except NotFoundError:
                pass

            delta = datetime.now() - start
            if delta >= timeout:
                raise QueryTimeoutError()

            time.sleep(internal.total_seconds())

    def query_tx(self, tx_hash: str) -> TxResponse:
        req = GetTxRequest(hash=tx_hash)
        try:
            resp = self.txs.GetTx(req)
        except grpc.RpcError as e:
            details = e.details()
            if "not found" in details:
                raise NotFoundError()
            else:
                raise
        except RuntimeError as e:
            details = str(e)
            if "tx" in details and "not found" in details:
                raise NotFoundError()
            else:
                raise

        return self._parse_tx_response(resp.tx_response)

    @staticmethod
    def _parse_tx_response(tx_response: Any) -> TxResponse:
        # parse the transaction logs
        logs = []
        for log_data in tx_response.logs:
            events = {}
            for event in log_data.events:
                events[event.type] = {a.key: a.value for a in event.attributes}
            logs.append(
                MessageLog(
                    index=int(log_data.msg_index), log=log_data.msg_index, events=events
                )
            )

        # parse the transaction events
        events = {}
        for event in tx_response.events:
            event_data = events.get(event.type, {})
            for attribute in event.attributes:
                event_data[attribute.key.decode()] = attribute.value.decode()
            events[event.type] = event_data

        return TxResponse(
            hash=str(tx_response.txhash),
            height=int(tx_response.height),
            code=int(tx_response.code),
            gas_wanted=int(tx_response.gas_wanted),
            gas_used=int(tx_response.gas_used),
            raw_log=str(tx_response.raw_log),
            logs=logs,
            events=events,
        )

    def simulate_tx(self, tx: Transaction) -> int:
        if tx.state != TxState.Final:
            raise RuntimeError("Unable to simulate non final transaction")

        req = SimulateRequest(tx=tx.tx)
        resp = self.txs.Simulate(req)

        return int(resp.gas_info.gas_used)

    def broadcast_tx(self, tx: Transaction) -> SubmittedTx:

        # create the broadcast request
        broadcast_req = BroadcastTxRequest(
            tx_bytes=tx.tx.SerializeToString(), mode=BroadcastMode.BROADCAST_MODE_SYNC
        )

        # broadcast the transaction
        resp = self.txs.BroadcastTx(broadcast_req)
        tx_digest = resp.tx_response.txhash

        # check that the response is successful
        initial_tx_response = self._parse_tx_response(resp.tx_response)
        initial_tx_response.ensure_successful()

        return SubmittedTx(self, tx_digest)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: cosmos/gov/v1beta1/tx.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from cosmos.base.v1beta1 import coin_pb2 as cosmos_dot_base_dot_v1beta1_dot_coin__pb2
from cosmos.gov.v1beta1 import gov_pb2 as cosmos_dot_gov_dot_v1beta1_dot_gov__pb2
from cosmos_proto import cosmos_pb2 as cosmos__proto_dot_cosmos__pb2
from gogoproto import gogo_pb2 as gogoproto_dot_gogo__pb2
from google.protobuf import any_pb2 as google_dot_protobuf_dot_any__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='cosmos/gov/v1beta1/tx.proto',
  package='cosmos.gov.v1beta1',
  syntax='proto3',
  serialized_options=b'Z(github.com/cosmos/cosmos-sdk/x/gov/types',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x1b\x63osmos/gov/v1beta1/tx.proto\x12\x12\x63osmos.gov.v1beta1\x1a\x1e\x63osmos/base/v1beta1/coin.proto\x1a\x1c\x63osmos/gov/v1beta1/gov.proto\x1a\x19\x63osmos_proto/cosmos.proto\x1a\x14gogoproto/gogo.proto\x1a\x19google/protobuf/any.proto\"\xeb\x01\n\x11MsgSubmitProposal\x12\x32\n\x07\x63ontent\x18\x01 \x01(\x0b\x32\x14.google.protobuf.AnyB\x0b\xca\xb4-\x07\x43ontent\x12~\n\x0finitial_deposit\x18\x02 \x03(\x0b\x32\x19.cosmos.base.v1beta1.CoinBJ\xc8\xde\x1f\x00\xaa\xdf\x1f(github.com/cosmos/cosmos-sdk/types.Coins\xf2\xde\x1f\x16yaml:\"initial_deposit\"\x12\x10\n\x08proposer\x18\x03 \x01(\t:\x10\xe8\xa0\x1f\x00\x98\xa0\x1f\x00\x80\xdc \x00\x88\xa0\x1f\x00\"W\n\x19MsgSubmitProposalResponse\x12:\n\x0bproposal_id\x18\x01 \x01(\x04\x42%\xea\xde\x1f\x0bproposal_id\xf2\xde\x1f\x12yaml:\"proposal_id\"\"\x96\x01\n\x07MsgVote\x12:\n\x0bproposal_id\x18\x01 \x01(\x04\x42%\xea\xde\x1f\x0bproposal_id\xf2\xde\x1f\x12yaml:\"proposal_id\"\x12\r\n\x05voter\x18\x02 \x01(\t\x12.\n\x06option\x18\x03 \x01(\x0e\x32\x1e.cosmos.gov.v1beta1.VoteOption:\x10\xe8\xa0\x1f\x00\x98\xa0\x1f\x00\x80\xdc \x00\x88\xa0\x1f\x00\"\x11\n\x0fMsgVoteResponse\"\x9e\x01\n\x0fMsgVoteWeighted\x12+\n\x0bproposal_id\x18\x01 \x01(\x04\x42\x16\xf2\xde\x1f\x12yaml:\"proposal_id\"\x12\r\n\x05voter\x18\x02 \x01(\t\x12=\n\x07options\x18\x03 \x03(\x0b\x32&.cosmos.gov.v1beta1.WeightedVoteOptionB\x04\xc8\xde\x1f\x00:\x10\xe8\xa0\x1f\x00\x98\xa0\x1f\x00\x80\xdc \x00\x88\xa0\x1f\x00\"\x19\n\x17MsgVoteWeightedResponse\"\xca\x01\n\nMsgDeposit\x12:\n\x0bproposal_id\x18\x01 \x01(\x04\x42%\xea\xde\x1f\x0bproposal_id\xf2\xde\x1f\x12yaml:\"proposal_id\"\x12\x11\n\tdepositor\x18\x02 \x01(\t\x12[\n\x06\x61mount\x18\x03 \x03(\x0b\x32\x19.cosmos.base.v1beta1.CoinB0\xc8\xde\x1f\x00\xaa\xdf\x1f(github.com/cosmos/cosmos-sdk/types.Coins:\x10\xe8\xa0\x1f\x00\x98\xa0\x1f\x00\x80\xdc \x00\x88\xa0\x1f\x00\"\x14\n\x12MsgDepositResponse2\xec\x02\n\x03Msg\x12\x66\n\x0eSubmitProposal\x12%.cosmos.gov.v1beta1.MsgSubmitProposal\x1a-.cosmos.gov.v1beta1.MsgSubmitProposalResponse\x12H\n\x04Vote\x12\x1b.cosmos.gov.v1beta1.MsgVote\x1a#.cosmos.gov.v1beta1.MsgVoteResponse\x12`\n\x0cVoteWeighted\x12#.cosmos.gov.v1beta1.MsgVoteWeighted\x1a+.cosmos.gov.v1beta1.MsgVoteWeightedResponse\x12Q\n\x07\x44\x65posit\x12\x1e.cosmos.gov.v1beta1.MsgDeposit\x1a&.cosmos.gov.v1beta1.MsgDepositResponseB*Z(github.com/cosmos/cosmos-sdk/x/gov/typesb\x06proto3'
  ,
  dependencies=[cosmos_dot_base_dot_v1beta1_dot_coin__pb2.DESCRIPTOR,cosmos_dot_gov_dot_v1beta1_dot_gov__pb2.DESCRIPTOR,cosmos__proto_dot_cosmos__pb2.DESCRIPTOR,gogoproto_dot_gogo__pb2.DESCRIPTOR,google_dot_protobuf_dot_any__pb2.DESCRIPTOR,])




_MSGSUBMITPROPOSAL = _descriptor.Descriptor(
  name='MsgSubmitProposal',
  full_name='cosmos.gov.v1beta1.MsgSubmitProposal',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='content', full_name='cosmos.gov.v1beta1.MsgSubmitProposal.content', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\312\264-\007Content', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='initial_deposit', full_name='cosmos.gov.v1beta1.MsgSubmitProposal.initial_deposit', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\310\336\037\000\252\337\037(github.com/cosmos/cosmos-sdk/types.Coins\362\336\037\026yaml:\"initial_deposit\"', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='proposer', full_name='cosmos.gov.v1beta1.MsgSubmitProposal.proposer', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'\350\240\037\000\230\240\037\000\200\334 \000\210\240\037\000',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=190,
  serialized_end=425,
)


_MSGSUBMITPROPOSALRESPONSE = _descriptor.Descriptor(
  name='MsgSubmitProposalResponse',
  full_name='cosmos.gov.v1beta1.MsgSubmitProposalResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='proposal_id', full_name='cosmos.gov.v1beta1.MsgSubmitProposalResponse.proposal_id', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\352\336\037\013proposal_id\362\336\037\022yaml:\"proposal_id\"', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=427,
  serialized_end=514,
)


_MSGVOTE = _descriptor.Descriptor(
  name='MsgVote',
  full_name='cosmos.gov.v1beta1.MsgVote',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='proposal_id', full_name='cosmos.gov.v1beta1.MsgVote.proposal_id', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\352\336\037\013proposal_id\362\336\037\022yaml:\"proposal_id\"', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='voter', full_name='cosmos.gov.v1beta1.MsgVote.voter', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='option', full_name='cosmos.gov.v1beta1.MsgVote.option', index=2,
      number=3, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'\350\240\037\000\230\240\037\000\200\334 \000\210\240\037\000',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=517,
  serialized_end=667,
)


_MSGVOTERESPONSE = _descriptor.Descriptor(
  name='MsgVoteResponse',
  full_name='cosmos.gov.v1beta1.MsgVoteResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=669,
  serialized_end=686,
)


_MSGVOTEWEIGHTED = _descriptor.Descriptor(
  name='MsgVoteWeighted',
  full_name='cosmos.gov.v1beta1.MsgVoteWeighted',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='proposal_id', full_name='cosmos.gov.v1beta1.MsgVoteWeighted.proposal_id', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\362\336\037\022yaml:\"proposal_id\"', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='voter', full_name='cosmos.gov.v1beta1.MsgVoteWeighted.voter', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='options', full_name='cosmos.gov.v1beta1.MsgVoteWeighted.options', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\310\336\037\000', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'\350\240\037\000\230\240\037\000\200\334 \000\210\240\037\000',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=689,
  serialized_end=847,
)


_MSGVOTEWEIGHTEDRESPONSE = _descriptor.Descriptor(
  name='MsgVoteWeightedResponse',
  full_name='cosmos.gov.v1beta1.MsgVoteWeightedResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=849,
  serialized_end=874,
)


_MSGDEPOSIT = _descriptor.Descriptor(
  name='MsgDeposit',
  full_name='cosmos.gov.v1beta1.MsgDeposit',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='proposal_id', full_name='cosmos.gov.v1beta1.MsgDeposit.proposal_id', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\352\336\037\013proposal_id\362\336\037\022yaml:\"proposal_id\"', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='depositor', full_name='cosmos.gov.v1beta1.MsgDeposit.depositor', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='amount', full_name='cosmos.gov.v1beta1.MsgDeposit.amount', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\310\336\037\000\252\337\037(github.com/cosmos/cosmos-sdk/types.Coins', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'\350\240\037\000\230\240\037\000\200\334 \000\210\240\037\000',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=877,
  serialized_end=1079,
)


_MSGDEPOSITRESPONSE = _descriptor.Descriptor(
  name='MsgDepositResponse',
  full_name='cosmos.gov.v1beta1.MsgDepositResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1081,
  serialized_end=1101,
)

_MSGSUBMITPROPOSAL.fields_by_name['content'].message_type = google_dot_protobuf_dot_any__pb2._ANY
_MSGSUBMITPROPOSAL.fields_by_name['initial_deposit'].message_type = cosmos_dot_base_dot_v1beta1_dot_coin__pb2._COIN
_MSGVOTE.fields_by_name['option'].enum_type = cosmos_dot_gov_dot_v1beta1_dot_gov__pb2._VOTEOPTION
_MSGVOTEWEIGHTED.fields_by_name['options'].message_type = cosmos_dot_gov_dot_v1beta1_dot_gov__pb2._WEIGHTEDVOTEOPTION
_MSGDEPOSIT.fields_by_name['amount'].message_type = cosmos_dot_base_dot_v1beta1_dot_coin__pb2._COIN
DESCRIPTOR.message_types_by_name['MsgSubmitProposal'] = _MSGSUBMITPROPOSAL
DESCRIPTOR.message_types_by_name['MsgSubmitProposalResponse'] = _MSGSUBMITPROPOSALRESPONSE
DESCRIPTOR.message_types_by_name['MsgVote'] = _MSGVOTE
DESCRIPTOR.message_types_by_name['MsgVoteResponse'] = _MSGVOTERESPONSE
DESCRIPTOR.message_types_by_name['MsgVoteWeighted'] = _MSGVOTEWEIGHTED
DESCRIPTOR.message_types_by_name['MsgVoteWeightedResponse'] = _MSGVOTEWEIGHTEDRESPONSE
DESCRIPTOR.message_types_by_name['MsgDeposit'] = _MSGDEPOSIT
DESCRIPTOR.message_types_by_name['MsgDepositResponse'] = _MSGDEPOSITRESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

MsgSubmitProposal = _reflection.GeneratedProtocolMessageType('MsgSubmitProposal', (_message.Message,), {
  'DESCRIPTOR' : _MSGSUBMITPROPOSAL,
  '__module__' : 'cosmos.gov.v1beta1.tx_pb2'
  # @@protoc_insertion_point(class_scope:cosmos.gov.v1beta1.MsgSubmitProposal)
  })
_sym_db.RegisterMessage(MsgSubmitProposal)

MsgSubmitProposalResponse = _reflection.GeneratedProtocolMessageType('MsgSubmitProposalResponse', (_message.Message,), {
  'DESCRIPTOR' : _MSGSUBMITPROPOSALRESPONSE,
  '__module__' : 'cosmos.gov.v1beta1.tx_pb2'
  # @@protoc_insertion_point(class_scope:cosmos.gov.v1beta1.MsgSubmitProposalResponse)
  })
_sym_db.RegisterMessage(MsgSubmitProposalResponse)

MsgVote = _reflection.GeneratedProtocolMessageType('MsgVote', (_message.Message,), {
  'DESCRIPTOR' : _MSGVOTE,
  '__module__' : 'cosmos.gov.v1beta1.tx_pb2'
  # @@protoc_insertion_point(class_scope:cosmos.gov.v1beta1.MsgVote)
  })
_sym_db.RegisterMessage(MsgVote)

MsgVoteResponse = _reflection.GeneratedProtocolMessageType('MsgVoteResponse', (_message.Message,), {
  'DESCRIPTOR' : _MSGVOTERESPONSE,
  '__module__' : 'cosmos.gov.v1beta1.tx_pb2'
  # @@protoc_insertion_point(class_scope:cosmos.gov.v1beta1.MsgVoteResponse)
  })
_sym_db.RegisterMessage(MsgVoteResponse)

MsgVoteWeighted = _reflection.GeneratedProtocolMessageType('MsgVoteWeighted', (_message.Message,), {
  'DESCRIPTOR' : _MSGVOTEWEIGHTED,
  '__module__' : 'cosmos.gov.v1beta1.tx_pb2'
  # @@protoc_insertion_point(class_scope:cosmos.gov.v1beta1.MsgVoteWeighted)
  })
_sym_db.RegisterMessage(MsgVoteWeighted)

MsgVoteWeightedResponse = _reflection.GeneratedProtocolMessageType('MsgVoteWeightedResponse', (_message.Message,), {
  'DESCRIPTOR' : _MSGVOTEWEIGHTEDRESPONSE,
  '__module__' : 'cosmos.gov.v1beta1.tx_pb2'
  # @@protoc_insertion_point(class_scope:cosmos.gov.v1beta1.MsgVoteWeightedResponse)
  })
_sym_db.RegisterMessage(MsgVoteWeightedResponse)

MsgDeposit = _reflection.GeneratedProtocolMessageType('MsgDeposit', (_message.Message,), {
  'DESCRIPTOR' : _MSGDEPOSIT,
  '__module__' : 'cosmos.gov.v1beta1.tx_pb2'
  # @@protoc_insertion_point(class_scope:cosmos.gov.v1beta1.MsgDeposit)
  })
_sym_db.RegisterMessage(MsgDeposit)

MsgDepositResponse = _reflection.GeneratedProtocolMessageType('MsgDepositResponse', (_message.Message,), {
  'DESCRIPTOR' : _MSGDEPOSITRESPONSE,
  '__module__' : 'cosmos.gov.v1beta1.tx_pb2'
  # @@protoc_insertion_point(class_scope:cosmos.gov.v1beta1.MsgDepositResponse)
  })
_sym_db.RegisterMessage(MsgDepositResponse)


DESCRIPTOR._options = None
_MSGSUBMITPROPOSAL.fields_by_name['content']._options = None
_MSGSUBMITPROPOSAL.fields_by_name['initial_deposit']._options = None
_MSGSUBMITPROPOSAL._options = None
_MSGSUBMITPROPOSALRESPONSE.fields_by_name['proposal_id']._options = None
_MSGVOTE.fields_by_name['proposal_id']._options = None
_MSGVOTE._options = None
_MSGVOTEWEIGHTED.fields_by_name['proposal_id']._options = None
_MSGVOTEWEIGHTED.fields_by_name['options']._options = None
_MSGVOTEWEIGHTED._options = None
_MSGDEPOSIT.fields_by_name['proposal_id']._options = None
_MSGDEPOSIT.fields_by_name['amount']._options = None
_MSGDEPOSIT._options = None

_MSG = _descriptor.ServiceDescriptor(
  name='Msg',
  full_name='cosmos.gov.v1beta1.Msg',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=1104,
  serialized_end=1468,
  methods=[
  _descriptor.MethodDescriptor(
    name='SubmitProposal',
    full_name='cosmos.gov.v1beta1.Msg.SubmitProposal',
    index=0,
    containing_service=None,
    input_type=_MSGSUBMITPROPOSAL,
    output_type=_MSGSUBMITPROPOSALRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='Vote',
    full_name='cosmos.gov.v1beta1.Msg.Vote',
    index=1,
    containing_service=None,
    input_type=_MSGVOTE,
    output_type=_MSGVOTERESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='VoteWeighted',
    full_name='cosmos.gov.v1beta1.Msg.VoteWeighted',
    index=2,
    containing_service=None,
    input_type=_MSGVOTEWEIGHTED,
    output_type=_MSGVOTEWEIGHTEDRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='Deposit',
    full_name='cosmos.gov.v1beta1.Msg.Deposit',
    index=3,
    containing_service=None,
    input_type=_MSGDEPOSIT,
    output_type=_MSGDEPOSITRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_MSG)

DESCRIPTOR.services_by_name['Msg'] = _MSG

# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2018-2022 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------
"""Tests for REST implementation of Cosmos Base Tendermint."""
import json
from typing import Dict, Tuple
from unittest import TestCase

from google.protobuf.json_format import ParseDict
from google.protobuf.wrappers_pb2 import Int32Value  # noqa # needed for protobuf decode

from cosmpy.protos.cosmos.base.tendermint.v1beta1.query_pb2 import (
    GetBlockByHeightRequest,
    GetBlockByHeightResponse,
    GetLatestBlockRequest,
    GetLatestBlockResponse,
    GetLatestValidatorSetRequest,
    GetLatestValidatorSetResponse,
    GetNodeInfoRequest,
    GetNodeInfoResponse,
    GetSyncingRequest,
    GetSyncingResponse,
    GetValidatorSetByHeightRequest,
    GetValidatorSetByHeightResponse,
)
from cosmpy.tendermint.rest_client import CosmosBaseTendermintRestClient
from tests.helpers import MockRestClient

TYPE = {
    "@type": "type.googleapis.com/google.protobuf.Int32Value",
    "value": "42",
}


class CosmosBaseTendermintRestClientTestCase(TestCase):
    """Test case for CosmosBaseTendermintRestClient class."""

    REST_CLIENT = CosmosBaseTendermintRestClient

    def make_clients(
        self, response_content: Dict
    ) -> Tuple[MockRestClient, CosmosBaseTendermintRestClient]:
        """
        Make  mock client and rest client api for specific content.

        :param response_content: dict
        :return: rest client instance
        """
        mock_client = MockRestClient(json.dumps(response_content).encode("utf-8"))
        rest_client = self.REST_CLIENT(mock_client)
        return mock_client, rest_client

    def test_GetNodeInfo(self):
        """Test GetNodeInfo method."""
        content = {
            "default_node_info": {
                "protocol_version": {
                    "p2p": "12",
                    "block": "12",
                    "app": "12",
                },
                "default_node_id": "string",
                "listen_addr": "string",
                "network": "string",
                "version": "string",
                "channels": "string",
                "moniker": "string",
                "other": {"tx_index": "string", "rpc_address": "string"},
            },
            "application_version": {
                "name": "string",
                "app_name": "string",
                "version": "string",
                "git_commit": "string",
                "build_tags": "string",
                "go_version": "string",
                "build_deps": [{"path": "string", "version": "string", "sum": "12"}],
            },
        }
        mock_client, rest_client = self.make_clients(content)
        expected_response = ParseDict(content, GetNodeInfoResponse())

        assert rest_client.GetNodeInfo(GetNodeInfoRequest()) == expected_response
        assert mock_client.last_base_url == "/cosmos/base/tendermint/v1beta1/node_info"

    def test_GetSyncing(self):
        """Test GetSyncing method."""
        content = {"syncing": True}
        mock_client, rest_client = self.make_clients(content)
        expected_response = ParseDict(content, GetSyncingResponse())

        assert rest_client.GetSyncing(GetSyncingRequest()) == expected_response
        assert mock_client.last_base_url == "/cosmos/base/tendermint/v1beta1/syncing"

    def test_GetLatestBlock(self):
        """Test GetLatestBlock method."""
        content = {
            "block_id": {
                "hash": "string",
                "part_set_header": {"total": 0, "hash": "string"},
            },
            "block": {
                "header": {
                    "version": {"block": "12", "app": "12"},
                    "chain_id": "string",
                    "height": "12",
                    "time": "2022-03-29T10:21:54.568Z",
                    "last_block_id": {
                        "hash": "string",
                        "part_set_header": {"total": 0, "hash": "string"},
                    },
                    "last_commit_hash": "string",
                    "data_hash": "string",
                    "validators_hash": "string",
                    "next_validators_hash": "string",
                    "consensus_hash": "string",
                    "app_hash": "string",
                    "last_results_hash": "string",
                    "evidence_hash": "string",
                    "proposer_address": "string",
                },
                "data": {"txs": ["string"]},
                "evidence": {"evidence": []},
                "last_commit": {
                    "height": "12",
                    "round": 0,
                    "block_id": {
                        "hash": "string",
                        "part_set_header": {"total": 0, "hash": "string"},
                    },
                    "signatures": [
                        {
                            "block_id_flag": "BLOCK_ID_FLAG_UNKNOWN",
                            "validator_address": "string",
                            "timestamp": "2022-03-29T10:21:54.569Z",
                            "signature": "string",
                        }
                    ],
                },
            },
        }
        mock_client, rest_client = self.make_clients(content)
        expected_response = ParseDict(content, GetLatestBlockResponse())

        assert rest_client.GetLatestBlock(GetLatestBlockRequest()) == expected_response
        assert (
            mock_client.last_base_url == "/cosmos/base/tendermint/v1beta1/blocks/latest"
        )

    def test_GetBlockByHeight(self):
        """Test GetBlockByHeight method."""
        content = {
            "block_id": {
                "hash": "string",
                "part_set_header": {"total": 0, "hash": "string"},
            },
            "block": {
                "header": {
                    "version": {"block": "12", "app": "12"},
                    "chain_id": "string",
                    "height": "12",
                    "time": "2022-03-29T10:27:01.686Z",
                    "last_block_id": {
                        "hash": "string",
                        "part_set_header": {"total": 0, "hash": "string"},
                    },
                    "last_commit_hash": "string",
                    "data_hash": "string",
                    "validators_hash": "string",
                    "next_validators_hash": "string",
                    "consensus_hash": "string",
                    "app_hash": "string",
                    "last_results_hash": "string",
                    "evidence_hash": "string",
                    "proposer_address": "string",
                },
                "data": {"txs": ["string"]},
                "evidence": {"evidence": []},
                "last_commit": {
                    "height": "12",
                    "round": 0,
                    "block_id": {
                        "hash": "string",
                        "part_set_header": {"total": 0, "hash": "string"},
                    },
                    "signatures": [
                        {
                            "block_id_flag": "BLOCK_ID_FLAG_UNKNOWN",
                            "validator_address": "string",
                            "timestamp": "2022-03-29T10:27:01.687Z",
                            "signature": "string",
                        }
                    ],
                },
            },
        }
        mock_client, rest_client = self.make_clients(content)
        expected_response = ParseDict(content, GetBlockByHeightResponse())

        assert (
            rest_client.GetBlockByHeight(GetBlockByHeightRequest(height=123))
            == expected_response
        )
        assert mock_client.last_base_url == "/cosmos/base/tendermint/v1beta1/blocks/123"

    def test_GetLatestValidatorSet(self):
        """Test GetLatestValidatorSet method."""
        content = {
            "block_height": "12",
            "validators": [
                {
                    "address": "string",
                    "pub_key": TYPE,
                    "voting_power": "12",
                    "proposer_priority": "12",
                }
            ],
            "pagination": {"next_key": "string", "total": "12"},
        }
        mock_client, rest_client = self.make_clients(content)
        expected_response = ParseDict(content, GetLatestValidatorSetResponse())

        assert (
            rest_client.GetLatestValidatorSet(GetLatestValidatorSetRequest())
            == expected_response
        )
        assert (
            mock_client.last_base_url
            == "/cosmos/base/tendermint/v1beta1/validatorsets/latest"
        )

    def test_GetValidatorSetByHeight(self):
        """Test GetValidatorSetByHeight method."""
        content = {
            "block_height": "12",
            "validators": [
                {
                    "address": "string",
                    "pub_key": TYPE,
                    "voting_power": "12",
                    "proposer_priority": "12",
                }
            ],
            "pagination": {"next_key": "string", "total": "12"},
        }
        mock_client, rest_client = self.make_clients(content)
        expected_response = ParseDict(content, GetValidatorSetByHeightResponse())

        assert (
            rest_client.GetValidatorSetByHeight(
                GetValidatorSetByHeightRequest(height=123)
            )
            == expected_response
        )
        assert (
            mock_client.last_base_url
            == "/cosmos/base/tendermint/v1beta1/validatorsets/123"
        )



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2018-2021 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------
import json

from cosmpy.aerial.client import LedgerClient
from cosmpy.aerial.client.distribution import create_withdraw_delegator_reward
from cosmpy.aerial.client.staking import create_delegate_msg
from cosmpy.aerial.config import NetworkConfig
from cosmpy.aerial.tx import SigningCfg, Transaction
from cosmpy.aerial.wallet import LocalWallet
from cosmpy.crypto.address import Address
from cosmpy.crypto.keypairs import PrivateKey
from cosmpy.protos.cosmos.bank.v1beta1.query_pb2 import QueryTotalSupplyRequest
from cosmpy.protos.cosmos.params.v1beta1.query_pb2 import QueryParamsRequest
from cosmpy.protos.cosmos.staking.v1beta1.query_pb2 import QueryValidatorsRequest


# This function returns the total reward for given:
# * f -> fee
# * S -> Initial Stake
# * k -> Reward Rate
# * D -> Total staking period
# * x -> Compounding Period
def M(x, f, S, k, D):
    return (S * (1 + (k * x)) ** (D / x)) + (
        (1 - ((1 + (k * x)) ** (D / x))) / (k * x)
    ) * f


def main():
    ledger = LedgerClient(NetworkConfig.fetchai_stable_testnet())

    # Set initial stake and desired stake period
    initial_stake = 50000000000000000000
    total_period = 60000

    req = QueryValidatorsRequest()
    resp = ledger.staking.Validators(req)

    # Calculate the total staked in the testnet

    total_stake = 0
    # validator.status == 3 refers to bonded validators
    validators_stake = [
        int(validator.tokens) for validator in resp.validators if validator.status == 3
    ]
    total_stake = sum(validators_stake)

    # Get validators comissions
    validators_comission = [
        int(validator.commission.commission_rates.rate)
        for validator in resp.validators
        if validator.status == 3
    ]

    validators = ledger.query_validators()
    validator = "not_selected"

    # Choose a threshold for a validators minimum percentage of total stake delegated
    stake_threshold = 0.10

    for i in range(len(validators_comission)):

        # Choose validator with lower commission
        validator_index = validators_comission.index(min(validators_comission))

        # Verify that it meets the minimum % threshold
        validator_stake_pct = validators_stake[validator_index] / total_stake
        if validator_stake_pct >= stake_threshold:

            # Set the selected validator
            validator = validators[validator_index]
            break

        else:
            # We omit this validator by setting his commssion to infinity
            validators_comission[validator_index] = float("inf")

    if validator == "not_selected":
        # Restart validators_comission list with oiriginal values
        validators_comission = [
            int(validator.commission.commission_rates.rate)
            for validator in resp.validators
            if validator.status == 3
        ]

        print("No validator meets the minium stake threshold requirement")

        # Proceed to select the validator with lowest commission
        validator_index = validators_comission.index(min(validators_comission))
        validator = validators[validator_index]

    # Query validator commission
    commission = float(resp.validators[0].commission.commission_rates.rate) / 1e18

    # Set percentage delegated of total stake
    pct_delegated = initial_stake / total_stake

    # Estmate fees for claiming and delegating rewards

    # Use any address with tokens available
    key = PrivateKey("XZ5BZQcr+FNl2usnSIQYpXsGWvBxKLRDkieUNIvMOV7=")
    alice = LocalWallet(key)
    alice_address = Address(key)._display

    tx = Transaction()

    # Add delegate msg
    tx.add_message(
        create_delegate_msg(alice_address, validator.address, initial_stake, "atestfet")
    )

    # Add claim reward msg
    tx.add_message(create_withdraw_delegator_reward(alice_address, validator.address))

    account = ledger.query_account(alice.address())

    tx.seal(
        SigningCfg.direct(alice.public_key(), account.sequence), fee="", gas_limit=0
    )
    tx.sign(alice.signer(), ledger.network_config.chain_id, account.number)
    tx.complete()

    # simulate the fee for the transaction
    _, str_tx_fee = ledger.estimate_gas_and_fee_for_tx(tx)

    denom = "atestfet"
    tx_fee = str_tx_fee[: -len(denom)]

    # Add a 20% to the fee estimation to get a more conservative estimate
    fee = int(tx_fee) * 1.20

    # Query chain variables

    # Total Supply of tokens
    req = QueryTotalSupplyRequest()
    resp = ledger.bank.TotalSupply(req)
    total_supply = float(json.loads(resp.supply[0].amount))

    # Inflation
    req = QueryParamsRequest(subspace="mint", key="InflationRate")
    resp = ledger.params.Params(req)
    inflation = float(json.loads(resp.param.value))

    # Community Tax
    req = QueryParamsRequest(subspace="distribution", key="communitytax")
    resp = ledger.params.Params(req)
    community_tax = float(json.loads(resp.param.value))

    # Anual reward calculation
    anual_reward = (
        (inflation * total_supply)
        * pct_delegated
        * (1 - community_tax)
        * (1 - commission)
    )

    # Convert from anual reward to minute reward
    minute_reward = anual_reward / 360 / 24 / 60
    rate = minute_reward / initial_stake

    # Compute optimal period
    f = fee
    S = initial_stake
    k = rate
    D = total_period

    # List of compounding periods
    X = [i for i in range(1, D)]

    # Evaluate function M on each compounding period
    R = [M(x, f, S, k, D) for x in X]

    # Fnd the period that maximizes rewards
    optimal_period = R.index(max(R)) + 1

    # These values can be used in aerial_compounder.py to maximize rewards
    print("total period: ", total_period, "minutes")
    print("optimal compounding period: ", optimal_period, "minutes")


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tendermint/types/evidence.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from gogoproto import gogo_pb2 as gogoproto_dot_gogo__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from tendermint.types import types_pb2 as tendermint_dot_types_dot_types__pb2
from tendermint.types import validator_pb2 as tendermint_dot_types_dot_validator__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='tendermint/types/evidence.proto',
  package='tendermint.types',
  syntax='proto3',
  serialized_options=b'Z7github.com/tendermint/tendermint/proto/tendermint/types',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x1ftendermint/types/evidence.proto\x12\x10tendermint.types\x1a\x14gogoproto/gogo.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x1ctendermint/types/types.proto\x1a tendermint/types/validator.proto\"\xb2\x01\n\x08\x45vidence\x12J\n\x17\x64uplicate_vote_evidence\x18\x01 \x01(\x0b\x32\'.tendermint.types.DuplicateVoteEvidenceH\x00\x12S\n\x1clight_client_attack_evidence\x18\x02 \x01(\x0b\x32+.tendermint.types.LightClientAttackEvidenceH\x00\x42\x05\n\x03sum\"\xd5\x01\n\x15\x44uplicateVoteEvidence\x12&\n\x06vote_a\x18\x01 \x01(\x0b\x32\x16.tendermint.types.Vote\x12&\n\x06vote_b\x18\x02 \x01(\x0b\x32\x16.tendermint.types.Vote\x12\x1a\n\x12total_voting_power\x18\x03 \x01(\x03\x12\x17\n\x0fvalidator_power\x18\x04 \x01(\x03\x12\x37\n\ttimestamp\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.TimestampB\x08\xc8\xde\x1f\x00\x90\xdf\x1f\x01\"\xfb\x01\n\x19LightClientAttackEvidence\x12\x37\n\x11\x63onflicting_block\x18\x01 \x01(\x0b\x32\x1c.tendermint.types.LightBlock\x12\x15\n\rcommon_height\x18\x02 \x01(\x03\x12\x39\n\x14\x62yzantine_validators\x18\x03 \x03(\x0b\x32\x1b.tendermint.types.Validator\x12\x1a\n\x12total_voting_power\x18\x04 \x01(\x03\x12\x37\n\ttimestamp\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.TimestampB\x08\xc8\xde\x1f\x00\x90\xdf\x1f\x01\"B\n\x0c\x45videnceList\x12\x32\n\x08\x65vidence\x18\x01 \x03(\x0b\x32\x1a.tendermint.types.EvidenceB\x04\xc8\xde\x1f\x00\x42\x39Z7github.com/tendermint/tendermint/proto/tendermint/typesb\x06proto3'
  ,
  dependencies=[gogoproto_dot_gogo__pb2.DESCRIPTOR,google_dot_protobuf_dot_timestamp__pb2.DESCRIPTOR,tendermint_dot_types_dot_types__pb2.DESCRIPTOR,tendermint_dot_types_dot_validator__pb2.DESCRIPTOR,])




_EVIDENCE = _descriptor.Descriptor(
  name='Evidence',
  full_name='tendermint.types.Evidence',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='duplicate_vote_evidence', full_name='tendermint.types.Evidence.duplicate_vote_evidence', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='light_client_attack_evidence', full_name='tendermint.types.Evidence.light_client_attack_evidence', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='sum', full_name='tendermint.types.Evidence.sum',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=173,
  serialized_end=351,
)


_DUPLICATEVOTEEVIDENCE = _descriptor.Descriptor(
  name='DuplicateVoteEvidence',
  full_name='tendermint.types.DuplicateVoteEvidence',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='vote_a', full_name='tendermint.types.DuplicateVoteEvidence.vote_a', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='vote_b', full_name='tendermint.types.DuplicateVoteEvidence.vote_b', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='total_voting_power', full_name='tendermint.types.DuplicateVoteEvidence.total_voting_power', index=2,
      number=3, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='validator_power', full_name='tendermint.types.DuplicateVoteEvidence.validator_power', index=3,
      number=4, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='timestamp', full_name='tendermint.types.DuplicateVoteEvidence.timestamp', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\310\336\037\000\220\337\037\001', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=354,
  serialized_end=567,
)


_LIGHTCLIENTATTACKEVIDENCE = _descriptor.Descriptor(
  name='LightClientAttackEvidence',
  full_name='tendermint.types.LightClientAttackEvidence',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='conflicting_block', full_name='tendermint.types.LightClientAttackEvidence.conflicting_block', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='common_height', full_name='tendermint.types.LightClientAttackEvidence.common_height', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='byzantine_validators', full_name='tendermint.types.LightClientAttackEvidence.byzantine_validators', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='total_voting_power', full_name='tendermint.types.LightClientAttackEvidence.total_voting_power', index=3,
      number=4, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='timestamp', full_name='tendermint.types.LightClientAttackEvidence.timestamp', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\310\336\037\000\220\337\037\001', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=570,
  serialized_end=821,
)


_EVIDENCELIST = _descriptor.Descriptor(
  name='EvidenceList',
  full_name='tendermint.types.EvidenceList',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='evidence', full_name='tendermint.types.EvidenceList.evidence', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\310\336\037\000', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=823,
  serialized_end=889,
)

_EVIDENCE.fields_by_name['duplicate_vote_evidence'].message_type = _DUPLICATEVOTEEVIDENCE
_EVIDENCE.fields_by_name['light_client_attack_evidence'].message_type = _LIGHTCLIENTATTACKEVIDENCE
_EVIDENCE.oneofs_by_name['sum'].fields.append(
  _EVIDENCE.fields_by_name['duplicate_vote_evidence'])
_EVIDENCE.fields_by_name['duplicate_vote_evidence'].containing_oneof = _EVIDENCE.oneofs_by_name['sum']
_EVIDENCE.oneofs_by_name['sum'].fields.append(
  _EVIDENCE.fields_by_name['light_client_attack_evidence'])
_EVIDENCE.fields_by_name['light_client_attack_evidence'].containing_oneof = _EVIDENCE.oneofs_by_name['sum']
_DUPLICATEVOTEEVIDENCE.fields_by_name['vote_a'].message_type = tendermint_dot_types_dot_types__pb2._VOTE
_DUPLICATEVOTEEVIDENCE.fields_by_name['vote_b'].message_type = tendermint_dot_types_dot_types__pb2._VOTE
_DUPLICATEVOTEEVIDENCE.fields_by_name['timestamp'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_LIGHTCLIENTATTACKEVIDENCE.fields_by_name['conflicting_block'].message_type = tendermint_dot_types_dot_types__pb2._LIGHTBLOCK
_LIGHTCLIENTATTACKEVIDENCE.fields_by_name['byzantine_validators'].message_type = tendermint_dot_types_dot_validator__pb2._VALIDATOR
_LIGHTCLIENTATTACKEVIDENCE.fields_by_name['timestamp'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_EVIDENCELIST.fields_by_name['evidence'].message_type = _EVIDENCE
DESCRIPTOR.message_types_by_name['Evidence'] = _EVIDENCE
DESCRIPTOR.message_types_by_name['DuplicateVoteEvidence'] = _DUPLICATEVOTEEVIDENCE
DESCRIPTOR.message_types_by_name['LightClientAttackEvidence'] = _LIGHTCLIENTATTACKEVIDENCE
DESCRIPTOR.message_types_by_name['EvidenceList'] = _EVIDENCELIST
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Evidence = _reflection.GeneratedProtocolMessageType('Evidence', (_message.Message,), {
  'DESCRIPTOR' : _EVIDENCE,
  '__module__' : 'tendermint.types.evidence_pb2'
  # @@protoc_insertion_point(class_scope:tendermint.types.Evidence)
  })
_sym_db.RegisterMessage(Evidence)

DuplicateVoteEvidence = _reflection.GeneratedProtocolMessageType('DuplicateVoteEvidence', (_message.Message,), {
  'DESCRIPTOR' : _DUPLICATEVOTEEVIDENCE,
  '__module__' : 'tendermint.types.evidence_pb2'
  # @@protoc_insertion_point(class_scope:tendermint.types.DuplicateVoteEvidence)
  })
_sym_db.RegisterMessage(DuplicateVoteEvidence)

LightClientAttackEvidence = _reflection.GeneratedProtocolMessageType('LightClientAttackEvidence', (_message.Message,), {
  'DESCRIPTOR' : _LIGHTCLIENTATTACKEVIDENCE,
  '__module__' : 'tendermint.types.evidence_pb2'
  # @@protoc_insertion_point(class_scope:tendermint.types.LightClientAttackEvidence)
  })
_sym_db.RegisterMessage(LightClientAttackEvidence)

EvidenceList = _reflection.GeneratedProtocolMessageType('EvidenceList', (_message.Message,), {
  'DESCRIPTOR' : _EVIDENCELIST,
  '__module__' : 'tendermint.types.evidence_pb2'
  # @@protoc_insertion_point(class_scope:tendermint.types.EvidenceList)
  })
_sym_db.RegisterMessage(EvidenceList)


DESCRIPTOR._options = None
_DUPLICATEVOTEEVIDENCE.fields_by_name['timestamp']._options = None
_LIGHTCLIENTATTACKEVIDENCE.fields_by_name['timestamp']._options = None
_EVIDENCELIST.fields_by_name['evidence']._options = None
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2018-2022 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------
"""Tests for REST implementation of Cosmos Upgrade."""
import json
from typing import Dict, Tuple
from unittest import TestCase

from google.protobuf.json_format import ParseDict
from google.protobuf.wrappers_pb2 import Int32Value  # noqa # needed for protobuf decode

from cosmpy.protos.cosmos.upgrade.v1beta1.query_pb2 import (
    QueryAppliedPlanRequest,
    QueryAppliedPlanResponse,
    QueryCurrentPlanRequest,
    QueryCurrentPlanResponse,
)
from cosmpy.upgrade.rest_client import CosmosUpgradeRestClient
from tests.helpers import MockRestClient


class CosmosUpgradeRestClientTestCase(TestCase):
    """Test case for CosmosUpgradeRestClient class."""

    REST_CLIENT = CosmosUpgradeRestClient

    def make_clients(
        self, response_content: Dict
    ) -> Tuple[MockRestClient, CosmosUpgradeRestClient]:
        """
        Make  mock client and rest client api for specific content.

        :param response_content: dict
        :return: rest client instance
        """
        mock_client = MockRestClient(json.dumps(response_content).encode("utf-8"))
        rest_client = self.REST_CLIENT(mock_client)
        return mock_client, rest_client

    def test_CurrentPlan(self):
        """Test CurrentPlan method."""
        content = {
            "plan": {
                "name": "string",
                "time": "2022-03-29T10:02:52.926Z",
                "height": "1",
                "info": "string",
            }
        }
        mock_client, rest_client = self.make_clients(content)
        expected_response = ParseDict(content, QueryCurrentPlanResponse())

        assert rest_client.CurrentPlan(QueryCurrentPlanRequest()) == expected_response
        assert mock_client.last_base_url == "/cosmos/upgrade/v1beta1/current_plan"

    def test_AppliedPlan(self):
        """Test AppliedPlan method."""
        content = {"height": "12"}
        mock_client, rest_client = self.make_clients(content)
        expected_response = ParseDict(content, QueryAppliedPlanResponse())

        assert (
            rest_client.AppliedPlan(QueryAppliedPlanRequest(name="test"))
            == expected_response
        )
        assert mock_client.last_base_url == "/cosmos/upgrade/v1beta1/applied_plan/test"



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2018-2021 Fetch.AI Limited
#   Modifications copyright (C) 2022 Cros-Nest
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------

"""Tests for REST implementation of Distribution."""

import json
from unittest import TestCase

from google.protobuf.json_format import ParseDict

from cosmpy.distribution.rest_client import DistributionRestClient
from cosmpy.protos.cosmos.distribution.v1beta1.query_pb2 import (
    QueryCommunityPoolResponse,
    QueryDelegationRewardsRequest,
    QueryDelegationRewardsResponse,
    QueryDelegationTotalRewardsRequest,
    QueryDelegationTotalRewardsResponse,
    QueryDelegatorValidatorsRequest,
    QueryDelegatorValidatorsResponse,
    QueryDelegatorWithdrawAddressRequest,
    QueryDelegatorWithdrawAddressResponse,
    QueryParamsResponse,
    QueryValidatorCommissionRequest,
    QueryValidatorCommissionResponse,
    QueryValidatorOutstandingRewardsRequest,
    QueryValidatorOutstandingRewardsResponse,
    QueryValidatorSlashesRequest,
    QueryValidatorSlashesResponse,
)
from tests.helpers import MockRestClient


class DistributionRestClientTestCase(TestCase):
    """Test case for DistributionRestClient class."""

    @staticmethod
    def test_CommunityPool():
        """Test CommunityPool method."""
        content = {"pool": [{"denom": "string", "amount": "123"}]}
        mock_client = MockRestClient(json.dumps(content).encode("utf8"))

        expected_response = ParseDict(content, QueryCommunityPoolResponse())

        distribution = DistributionRestClient(mock_client)

        assert distribution.CommunityPool() == expected_response
        assert (
            mock_client.last_base_url == "/cosmos/distribution/v1beta1/community_pool"
        )

    @staticmethod
    def test_DelegationTotalRewards():
        """Test DelegationTotalRewards method."""
        content = {
            "rewards": [
                {
                    "validator_address": "string",
                    "reward": [{"denom": "string", "amount": "123"}],
                }
            ],
            "total": [{"denom": "string", "amount": "123"}],
        }
        mock_client = MockRestClient(json.dumps(content))

        expected_response = ParseDict(content, QueryDelegationTotalRewardsResponse())

        distribution = DistributionRestClient(mock_client)

        assert (
            distribution.DelegationTotalRewards(
                QueryDelegationTotalRewardsRequest(delegator_address="delegator_addr")
            )
            == expected_response
        )
        assert (
            mock_client.last_base_url
            == "/cosmos/distribution/v1beta1/delegators/delegator_addr/rewards"
        )

    @staticmethod
    def test_DelegationRewards():
        """Test DelegationRewards method."""
        content = {"rewards": [{"denom": "string", "amount": "1234"}]}
        mock_client = MockRestClient(json.dumps(content))

        expected_response = ParseDict(content, QueryDelegationRewardsResponse())

        distribution = DistributionRestClient(mock_client)

        assert (
            distribution.DelegationRewards(
                QueryDelegationRewardsRequest(
                    delegator_address="delegator_addr",
                    validator_address="validator_addr",
                )
            )
            == expected_response
        )
        assert (
            mock_client.last_base_url
            == "/cosmos/distribution/v1beta1/delegators/delegator_addr/rewards/validator_addr"
        )

    @staticmethod
    def test_DelegatorValidators():
        """Test DelegatorValidators method."""
        content = {"validators": ["string"]}
        mock_client = MockRestClient(json.dumps(content))

        expected_response = ParseDict(content, QueryDelegatorValidatorsResponse())

        distribution = DistributionRestClient(mock_client)

        assert (
            distribution.DelegatorValidators(
                QueryDelegatorValidatorsRequest(delegator_address="delegator_addr")
            )
            == expected_response
        )
        assert (
            mock_client.last_base_url
            == "/cosmos/distribution/v1beta1/delegators/delegator_addr/validators"
        )

    @staticmethod
    def test_DelegatorWithdrawAddress():
        """Test DelegatorWithdrawAddress method."""
        content = {"withdraw_address": "string"}
        mock_client = MockRestClient(json.dumps(content))

        expected_response = ParseDict(content, QueryDelegatorWithdrawAddressResponse())

        distribution = DistributionRestClient(mock_client)

        assert (
            distribution.DelegatorWithdrawAddress(
                QueryDelegatorWithdrawAddressRequest(delegator_address="delegator_addr")
            )
            == expected_response
        )
        assert (
            mock_client.last_base_url
            == "/cosmos/distribution/v1beta1/delegators/delegator_addr/withdraw_address"
        )

    @staticmethod
    def test_Params():
        """Test Params method."""
        content = {
            "params": {
                "community_tax": "0.1",
                "base_proposer_reward": "0.2",
                "bonus_proposer_reward": "0.3",
                "withdraw_addr_enabled": True,
            }
        }
        mock_client = MockRestClient(json.dumps(content))

        expected_response = ParseDict(content, QueryParamsResponse())

        distribution = DistributionRestClient(mock_client)

        # Check that the parameters are in the right format
        assert expected_response.params.community_tax == "0.1"
        assert expected_response.params.base_proposer_reward == "0.2"
        assert expected_response.params.bonus_proposer_reward == "0.3"
        assert expected_response.params.withdraw_addr_enabled is True

        assert distribution.Params() == expected_response
        assert mock_client.last_base_url == "/cosmos/distribution/v1beta1/params"

    @staticmethod
    def test_ValidatorCommission():
        """Test ValidatorCommission method."""
        content = {
            "commission": {"commission": [{"denom": "string", "amount": "1234"}]}
        }
        mock_client = MockRestClient(json.dumps(content))

        expected_response = ParseDict(content, QueryValidatorCommissionResponse())

        distribution = DistributionRestClient(mock_client)

        assert (
            distribution.ValidatorCommission(
                QueryValidatorCommissionRequest(validator_address="validator_addr")
            )
            == expected_response
        )
        assert (
            mock_client.last_base_url
            == "/cosmos/distribution/v1beta1/validators/validator_addr/commission"
        )

    @staticmethod
    def test_ValidatorOutstandingRewards():
        """Test ValidatorOutstandingRewards method."""
        content = {"rewards": {"rewards": [{"denom": "string", "amount": "1234"}]}}
        mock_client = MockRestClient(json.dumps(content))

        expected_response = ParseDict(
            content, QueryValidatorOutstandingRewardsResponse()
        )

        distribution = DistributionRestClient(mock_client)

        assert (
            distribution.ValidatorOutstandingRewards(
                QueryValidatorOutstandingRewardsRequest(
                    validator_address="validator_addr"
                )
            )
            == expected_response
        )
        assert (
            mock_client.last_base_url
            == "/cosmos/distribution/v1beta1/validators/validator_addr/outstanding_rewards"
        )

    @staticmethod
    def test_ValidatorSlashes():
        """Test ValidatorSlashes method."""
        content = {
            "slashes": [{"validator_period": "1", "fraction": "1"}],
            "pagination": {"next_key": None, "total": "1"},
        }
        mock_client = MockRestClient(json.dumps(content))

        expected_response = ParseDict(content, QueryValidatorSlashesResponse())

        distribution = DistributionRestClient(mock_client)

        assert (
            distribution.ValidatorSlashes(
                QueryValidatorSlashesRequest(validator_address="validator_addr")
            )
            == expected_response
        )
        assert (
            mock_client.last_base_url
            == "/cosmos/distribution/v1beta1/validators/validator_addr/slashes"
        )
